/*
	BlockAverage plugin for Avisynth -- average each 4x4 block of Y values in a YV12 image

	Copyright (C) 2002-2003 Simon Walters, All Rights Reserved

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	The author can be contacted at:
	Simon Walters
	siwalters@hotmail.com

V0.1 -1st release

*/
#include <stdio.h>
#include "windows.h"
#include "avisynth.h"
#include "info.h"

#include "memory.h"




class BlockAverage : public GenericVideoFilter {
	int param1, param2, param3, param4;
	
	unsigned int YMaxTot;

public:
	BlockAverage(PClip _child,
		int _param1, int _param2, int _param3,int _param4,
		IScriptEnvironment* env) :
	GenericVideoFilter(_child),
		param1(_param1), param2(_param2),
		 param3(_param3), param4(_param4)

	{
	// parameter checking goes in here
	if (!vi.IsYV12())
			env->ThrowError("BlockAverage: input to filter must be in YV12 colorspace");
	YMaxTot =255;
	}

//    BlockAverage(PClip _child) : GenericVideoFilter(_child) {}
    PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
};



PVideoFrame __stdcall BlockAverage::GetFrame(int n, IScriptEnvironment* env) {
// This is the implementation of the GetFrame function.
// See the header definition for further info.

	PVideoFrame src = child->GetFrame(n, env);
   // Request frame 'n' from the child (source) clip.
	PVideoFrame dst = env->NewVideoFrame(vi);
   // Construct a frame based on the information of the current frame
   // contained in the "vi" struct.
	
	const unsigned char* srcp = src->GetReadPtr();
  // Request a Read pointer from the source frame.
  // This will return the position of the upperleft pixel in YUY2 images,
  // and return the lower-left pixel in RGB.
  // RGB images are stored upside-down in memory. 
  // You should still process images from line 0 to height.

	unsigned char* dstp = dst->GetWritePtr();
	// Request a Write pointer from the newly created destination image.
  // You can request a writepointer to images that have just been
  // created by NewVideoFrame. If you recieve a frame from PClip->GetFrame(...)
  // you must call env->MakeWritable(&frame) be recieve a valid write pointer.
	
	const int dst_pitch = dst->GetPitch();
  // Requests pitch (length of a line) of the destination image.
  // For more information on pitch see: http://www.avisynth.org/index.php?page=WorkingWithImages
	// (short version - pitch is always equal to or greater than width to allow for seriously fast assembly code)

	const int dst_width = dst->GetRowSize();
  // Requests rowsize (number of used bytes in a line.
  // See the link above for more information.

	const int dst_height = dst->GetHeight();
  // Requests the height of the destination image.

	const int src_pitch = src->GetPitch();
	const int src_width = src->GetRowSize();
	const int src_height = src->GetHeight();
	
	int w, h,x,y,average;

		
  for (h=0; h < src_height;h+=2) {		//for every other line							 
	  for (w = 0; w < src_width; w+=2) { //for every other pixel
          average = 0;
		  for (y=0;y<2;y++) {
				for (x=0;x<2;x++)
					average+= *(srcp + w + x + (y * dst_pitch));
		  }
		  average = (char)(average /4);
		  *(dstp + w) = average;
		  *(dstp + w + 1) = average;
		  *(dstp + w + dst_pitch) = average;
		  *(dstp + w + 1 + dst_pitch) = average;

	  }
		srcp = srcp + (src_pitch<<1);            // Add the pitch (note use of pitch and not width) of one line (in bytes) to the source image.
		dstp = dstp + (dst_pitch<<1);            // Add the pitch of one line (in bytes) to the destination.
	}
	// end copy src to dst

  // The preceding code deals with all colourspaces but only copies the Y (luma) information
  // of planar colourspaces. This new section of code deals with the U and V planes of planar
  // formats (e.g. YV12).  The code is functionally identical with the previous code above. 

	const int dst_pitchUV = dst->GetPitch(PLANAR_U);	// The pitch,height and width information
	const int dst_widthUV = dst->GetRowSize(PLANAR_U);	// is guaranted to be the same for both
	const int dst_heightUV = dst->GetHeight(PLANAR_U);	// the U and V planes so we only the U
	const int src_pitchUV = src->GetPitch(PLANAR_U);	// plane values and use them for V as
	const int src_widthUV = src->GetRowSize(PLANAR_U);	// well
	const int src_heightUV = src->GetHeight(PLANAR_U);	//

	//Copy U plane src to dst
	srcp = src->GetReadPtr(PLANAR_U);
	dstp = dst->GetWritePtr(PLANAR_U);

	for (h=0; h < src_heightUV;h++) {
		for (w = 0; w < src_widthUV; w++)
			*(dstp + w) = *(srcp + w);
		srcp = srcp + src_pitchUV;
		dstp = dstp + dst_pitchUV;
	}
	// end copy U plane src to dst

	//Copy V plane src to dst
	srcp = src->GetReadPtr(PLANAR_V);
	dstp = dst->GetWritePtr(PLANAR_V);

	for (h=0; h < src_heightUV;h++) {
		for (w = 0; w < src_widthUV; w++)
			*(dstp + w) = *(srcp + w);
		srcp = srcp + src_pitchUV;
		dstp = dstp + dst_pitchUV;
	}
	// end copy V plane src to dst

	
  // As we now are finished processing the image, we return the destination image.
	return dst;
}


AVSValue __cdecl Create_BlockAverage(AVSValue args, void* user_data, IScriptEnvironment* env) {
    return new BlockAverage(args[0].AsClip(),
		 args[1].AsInt(0),	// param1
		 args[2].AsInt(0),	// param2
		 args[3].AsInt(0),	// param3
		 args[4].AsInt(0),	// param4

		 env);
}


extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) {
    env->AddFunction("BlockAverage", "c[param1]i[param2]i[param3]i[param4]i", Create_BlockAverage, 0);
    return "`BlockAverage' BlockAverage plugin";
}

